# Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

# * The software is provided "as is" without any warranty, neither express
#   nor implied.
# * In no event will the author be held liable for any damage(s) arising
#   from the use of the software.
# * Redistribution of the software is allowed only in unmodified form.
# * Permission is granted to use the software solely for personal private
#   and non-commercial purposes.
# * An individuals use of the software in his or her capacity or function
#   as an agent, (independent) contractor, employee, member or officer of
#   a business, corporation or organization (commercial or non-commercial)
#   does not qualify as personal private and non-commercial purpose.
# * Without written approval from the author the software must not be used
#   for a business, for commercial, corporate, governmental, military or
#   organizational purposes of any kind, or in a commercial, corporate,
#   governmental, military or organizational environment of any kind.

.SUFFIXES:

test: DLLDUMMY.DLL
#	MSIEXEC.EXE /Y "$(MAKEDIR)\DLLDUMMY.DLL"
#	REGSVR32.EXE "$(MAKEDIR)\DLLDUMMY.DLL"
	RUNDLL32.EXE "$(MAKEDIR)\DLLDUMMY.DLL",* Vulnerability and Exploit Detector Module

sign: DLLDUMMY.DLL
# https://msdn.microsoft.com/en-us/library/aa388170.aspx
# https://msdn.microsoft.com/en-us/library/aa387764.aspx
# https://msdn.microsoft.com/en-us/library/aa375542.aspx
# https://msdn.microsoft.com/en-us/library/dd323764.aspx
	SIGNTOOL.EXE Sign /A /D "Vulnerability and Exploit Detector" /DU "https://skanthak.hier-im-netz.de/minesweeper.html" /N "Stefan Kanthak" /T "http://timestamp.digicert.com/" /V DLLDUMMY.DLL
#	SIGNTOOL.EXE Sign /AS /D "Vulnerability and Exploit Detector" /DU "https://skanthak.hier-im-netz.de/minesweeper.html" /FD SHA256 /N "Stefan Kanthak" /TD SHA256 /TR "http://timestamp.digicert.com/" /V DLLDUMMY.DLL

clobber: clean
	Erase DLLDUMMY.DLL

clean:
	Erase DLLDUMMY.OBJ DLLDUMMY.IMP DLLDUMMY.EXP
	Erase DLLDUMMY.DOS DLLDUMMY.RES DLLDUMMY.CVT
	Erase DLLDUMMY.RC  DLLDUMMY.H   DLLDUMMY_???.BIN

all: DLLDUMMY.DLL DLLDUMMY.MAK
	Call <<DLLDUMMY.CMD
@Echo Off

Rem DLLDUMMY.CMD

Rem Build (a testbed of) DLLs with exports forwarded to corresponding
Rem target DLLs in "%SystemRoot%\System32\", using import libraries
Rem and export files generated from the exports of the target DLLs.

Rem Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

SetLocal EnableExtensions
Ver 1>DLLDUMMY.TMP
For /F "Delims=[.] Skip=1 Tokens=2,3 UseBackQ" %%X In ("DLLDUMMY.TMP") Do Set OSVERSION=%%X.%%Y
Set OSVERSION=%OSVERSION:* =%
Del DLLDUMMY.TMP

For %%Z In ("%SystemRoot%\System32\*.DLL" "%SystemRoot%\System32\DownLevel\*.DLL" "%SystemRoot%\System32\WBEM\*.DLL") Do Call :DLLDUMMY "%%~Z"
EndLocal
Exit /B

:DLLDUMMY
If Exist "%~n1.DLL" Goto :EOF

Echo %~n1

LINK.EXE /DUMP /EXPORTS /OUT:"%~n1.TMP" "%~1" 1>"%~n1.LOG"
If ERRORLEVEL 1 Goto :EOF

Call :DEF "%~1" 1>"%~n1.DEF"

LINK.EXE /LIB /BREPRO /DEF:"%~n1.DEF" /NODEFAULTLIB /NOLOGO /OUT:"%~n1.IMP" 1>>"%~n1.LOG"
If ERRORLEVEL 1 Goto :EOF

LINK.EXE /LINK /ALLOWBIND:NO /BREPRO /DLL /DYNAMICBASE /ENTRY:_DllMainCRTStartup /LARGEADDRESSAWARE /LAST:.data /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NODEFAULTLIB /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:%OSVERSION% /OUT:"%~n1.DLL" /RELEASE /STUB:DLLDUMMY.DOS /SUBSYSTEM:WINDOWS /SWAPRUN:CD,NET /VERSION:1.0 DLLDUMMY.OBJ DLLDUMMY.CVT "%~n1.IMP" "%~n1.EXP" ADVAPI32.LIB KERNEL32.LIB USER32.LIB 1>>"%~n1.LOG"
Rem LINK.EXE /LINK /ALLOWBIND:NO /BREPRO /DEF:"%~n1.DEF" /DLL /DYNAMICBASE /ENTRY:_DllMainCRTStartup /IMPLIB:"%~n1.TMP" /LARGEADDRESSAWARE /LAST:.data /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NODEFAULTLIB /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:%OSVERSION% /OUT:"%~n1.DLL" /RELEASE /STUB:DLLDUMMY.DOS /SUBSYSTEM:WINDOWS /SWAPRUN:CD,NET /VERSION:1.0 DLLDUMMY.OBJ DLLDUMMY.CVT "%~n1.IMP" ADVAPI32.LIB KERNEL32.LIB USER32.LIB 1>>"%~n1.LOG"
If ERRORLEVEL 1 Goto :EOF

Del "%~n1.DEF" "%~n1.EXP" "%~n1.IMP" "%~n1.LOG" "%~n1.TMP"
Goto :EOF

:DEF
Echo LIBRARY "%~n1"
Echo.
Echo EXPORTS

SetLocal EnableDelayedExpansion
Set LIBRARY=%~dpn1
Set LIBRARY=!LIBRARY:%SystemRoot%\=!
For /F "Delims= Skip=16 UseBackQ" %%@ In ("%~n1.TMP") Do If "%%@" == "  Summary" (Goto :EOF) Else Call :EXPORTS "%%@"
EndLocal
Goto :EOF

:EXPORTS
SetLocal
Set LINE=%1
Set LINE=%LINE:<=^^^^^^^<%
Set LINE=%LINE:>=^^^^^^^>%
Set LINE=%LINE:~1,-1%
Set ORDINAL=%LINE:~4,7%
Set ORDINAL=%ORDINAL: =%
Set HINT=%LINE:~12,4%
Set RVA=%LINE:~17,8%
Set NAME=%LINE:~26%

If "%HINT%" == "    " Goto :ORDINAL
If "%NAME%" == "[NONAME]" Goto :ORDINAL
If "%RVA%" == "        " Goto :FORWARD

:NAME
Echo  %NAME%=%LIBRARY%.%NAME% @%ORDINAL%
EndLocal
Goto :EOF

:ORDINAL
Echo  @%ORDINAL%=%LIBRARY%.#%ORDINAL% @%ORDINAL% NONAME
EndLocal
Goto :EOF

:FORWARD
Set NAME=%NAME: (forwarded to ==%
Echo  %NAME:~0,-1% @%ORDINAL%
EndLocal
Goto :EOF
<<NOKEEP

DLLDUMMY.DLL: DLLDUMMY.OBJ DLLDUMMY.DOS DLLDUMMY.CVT
	LINK.EXE /LINK /ALLOWBIND:NO /BREPRO /DLL /DYNAMICBASE /ENTRY:_DllMainCRTStartup /IMPLIB:DLLDUMMY.IMP /LARGEADDRESSAWARE /LAST:.data /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NODEFAULTLIB /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:5.1 /OUT:$@ /RELEASE /STUB:DLLDUMMY.DOS /SUBSYSTEM:WINDOWS /SWAPRUN:CD,NET /TEST /VERSION:1.0 DLLDUMMY.OBJ DLLDUMMY.CVT ADVAPI32.LIB KERNEL32.LIB USER32.LIB

DLLDUMMY.CVT: DLLDUMMY.RES
	CVTRES.EXE /BREPRO /NOLOGO /OUT:$@ /READONLY $**

DLLDUMMY.OBJ: DLLDUMMY.H DLLDUMMY.MAK
	CL.EXE /Brepro /c /DCALLER /DEVENTLOG /DGSCOOKIE /DSOUND /DTHREAD /DUSERICON /Fo$@ /GA /GF /GS /Gw /Gy /nologo /O1 /Os /Oy- /Tc<< /W4 /we4013 /Zl
// Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/7f0aews7.aspx

#pragma comment(compiler)

#define STRICT
#define UNICODE
#define WIN32_LEAN_AND_MEAN

#define WINVER		0x0501
#define _WIN32_WINNT	0x0501

#include <windows.h>
#include <sddl.h>
#include <lmcons.h>

#include "$(MAKEDIR)\DLLDUMMY.H"		// generated from "DLLDUMMY.MC"

#ifndef LOAD_LIBRARY_SEARCH_SYSTEM32
#define LOAD_LIBRARY_SEARCH_SYSTEM32	0x00000800UL
#endif

#ifndef SECURITY_MANDATORY_LABEL_AUTHORITY
#define SECURITY_MANDATORY_LABEL_AUTHORITY		{0, 0, 0, 0, 0, 16}

#define SECURITY_MANDATORY_UNTRUSTED_RID		0UL
#define SECURITY_MANDATORY_LOW_RID			4096UL
#define SECURITY_MANDATORY_MEDIUM_RID			8192UL
#define SECURITY_MANDATORY_MEDIUM_PLUS_RID		(SECURITY_MANDATORY_MEDIUM_RID + 256UL)
#define SECURITY_MANDATORY_HIGH_RID			12288UL
#define SECURITY_MANDATORY_SYSTEM_RID			16384UL
#define SECURITY_MANDATORY_PROTECTED_PROCESS_RID	20480UL
#define SECURITY_MANDATORY_SECURE_PROCESS_RID		28672UL

#define TokenIntegrityLevel	25
#endif

#ifdef CALLER
// https://msdn.microsoft.com/en-us/library/bb204633.aspx

#ifndef CaptureStackBackTrace
#define CaptureStackBackTrace	RtlCaptureStackBackTrace
#endif

__declspec(dllimport)
WORD	WINAPI	CaptureStackBackTrace(DWORD   FramesToSkip,
		                      DWORD   FramesToCapture,
		                      LPVOID  *BackTrace,
		                      LPDWORD BackTraceHash);
#endif // CALLER

#ifdef GSCOOKIE
#ifndef STATUS_STACK_BUFFER_OVERRUN
#define STATUS_STACK_BUFFER_OVERRUN	0xC0000409L
#endif

#ifndef EXCEPTION_STACK_BUFFER_OVERRUN
#define EXCEPTION_STACK_BUFFER_OVERRUN	0xC0000409L
#endif

#ifndef _WIN64
#define DEFAULT_SECURITY_COOKIE	0xBB40E64EUL	// = 3141592654 = 10**9 * pi
#define PRIVATE_SECURITY_COOKIE	0xA205B064UL	// = 2718281828 = 10**9 * e
#define RESERVE_SECURITY_COOKIE	0xA2F9836EUL	// = 2**33 / pi
#define SPECIAL_SECURITY_COOKIE	0xBC5AB1B1UL	// = 2**33 / e
#else						// = 3141592653589793241 >> 16 = 10**18 / 2**16 * pi
#define DEFAULT_SECURITY_COOKIE	0x00002B992DDFA232ULL
						// = 2718281828459045235 >> 16 = 10**18 / 2**16 * e
#define PRIVATE_SECURITY_COOKIE	0x000025B946EBC0B3ULL
						// = 2**49 / pi
#define RESERVE_SECURITY_COOKIE	0x0000A2F9836E4E44ULL
						// = 2**49 / e
#define SPECIAL_SECURITY_COOKIE	0x0000BC5AB1B16779ULL
#endif

DWORD_PTR	__security_cookie = DEFAULT_SECURITY_COOKIE;

#ifndef _WIN64
// https://msdn.microsoft.com/en-us/library/9a89h429.aspx

extern	LPVOID	__safe_se_handler_table[];
extern	BYTE	__safe_se_handler_count;

// https://msdn.microsoft.com/en-us/library/ms680328.aspx

const	struct	_IMAGE_LOAD_CONFIG_DIRECTORY_32
{
	DWORD	Size;
	DWORD	TimeDateStamp;
	WORD	MajorVersion;
	WORD	MinorVersion;
	DWORD	GlobalFlagsClear;
	DWORD	GlobalFlagsSet;
	DWORD	CriticalSectionDefaultTimeout;
	DWORD	DeCommitFreeBlockThreshold;
	DWORD	DeCommitTotalFreeThreshold;
	DWORD	LockPrefixTable;
	DWORD	MaximumAllocationSize;
	DWORD	VirtualMemoryThreshold;
	DWORD	ProcessHeapFlags;
	DWORD	ProcessAffinityMask;
	WORD	CSDVersion;
#if 0
	WORD	Reserved1;
#else
	WORD	DependentLoadFlags;
#endif
	DWORD	EditList;
	DWORD	SecurityCookie;
	DWORD	SEHandlerTable;
	DWORD	SEHandlerCount;
						// Threshold 1
	DWORD	GuardCFCheckFunctionPointer;
	DWORD	GuardCFDispatchFunctionPointer;
	DWORD	GuardCFFunctionTable;
	DWORD	GuardCFFunctionCount;
	DWORD	GuardFlags;
} _load_config_used = {sizeof(_load_config_used),
                       'CVSM',			// = "MSVC" = 2005-10-19 14:08:13 UTC
                       _MSC_VER / 100,
                       _MSC_VER % 100,
                       0UL,
#if 0
	// https://msdn.microsoft.com/en-us/library/cc779664.aspx
	// https://msdn.microsoft.com/en-us/library/ff549646.aspx
                       FLG_ENABLE_CLOSE_EXCEPTIONS | FLG_ENABLE_HANDLE_EXCEPTIONS,
#else
                       0UL,
#endif
                       0UL,
                       0UL,
                       0UL,
                       0UL,
                       0UL,
                       0UL,
#if 0
	// https://msdn.microsoft.com/en-us/library/ms810603.aspx
                       HEAP_GENERATE_EXCEPTIONS,
#else
                       0UL,
#endif
                       0UL,
                       0U,
#if 1	// NOTE: evaluated only on Windows 10 1607 "Anniversary Update"
	//       alias 'Redstone 1' and later versions, when value of
	//       'Size' member is 92 or greater!

	// https://msdn.microsoft.com/en-us/library/ms684179.aspx
                       LOAD_LIBRARY_SEARCH_SYSTEM32,
#else
                       0U,
#endif
                       0UL,
                       &__security_cookie,
                       __safe_se_handler_table,
                       &__safe_se_handler_count,
                       0UL,
                       0UL,
                       0UL,
                       0UL,
                       0UL};
#else
// https://msdn.microsoft.com/en-us/library/ms680328.aspx

const	struct	_IMAGE_LOAD_CONFIG_DIRECTORY_64
{
	DWORD	Size;
	DWORD	TimeDateStamp;
	WORD	MajorVersion;
	WORD	MinorVersion;
	DWORD	GlobalFlagsClear;
	DWORD	GlobalFlagsSet;
	DWORD	CriticalSectionDefaultTimeout;
	DWORD64	DeCommitFreeBlockThreshold;
	DWORD64	DeCommitTotalFreeThreshold;
	DWORD64	LockPrefixTable;
	DWORD64	MaximumAllocationSize;
	DWORD64	VirtualMemoryThreshold;
	DWORD64	ProcessAffinityMask;
	DWORD	ProcessHeapFlags;
	WORD	CSDVersion;
#if 0
	WORD	Reserved1;
#else
	WORD	DependentLoadFlags;
#endif
	DWORD64	EditList;
	DWORD64	SecurityCookie;
	DWORD64	SEHandlerTable;
	DWORD64	SEHandlerCount;
						// Threshold 1
	DWORD64	GuardCFCheckFunctionPointer;
	DWORD64	GuardCFDispatchFunctionPointer;
	DWORD64	GuardCFFunctionTable;
	DWORD64	GuardCFFunctionCount;
	DWORD	GuardFlags;
} _load_config_used = {sizeof(_load_config_used),
                       'CVSM',			// = "MSVC" = 2005-10-19 14:08:13 UTC
                       _MSC_VER / 100,
                       _MSC_VER % 100,
                       0UL,
#if 0
	// https://msdn.microsoft.com/en-us/library/cc779664.aspx
	// https://msdn.microsoft.com/en-us/library/ff549646.aspx
                       FLG_ENABLE_CLOSE_EXCEPTIONS | FLG_ENABLE_HANDLE_EXCEPTIONS,
#else
                       0UL,
#endif
                       0UL,
                       0ULL,
                       0ULL,
                       0ULL,
                       0ULL,
                       0ULL,
                       0ULL,
#if 0
	// https://msdn.microsoft.com/en-us/library/ms810603.aspx
                       HEAP_GENERATE_EXCEPTIONS,
#else
                       0UL,
#endif
                       0U,
#if 1	// NOTE: evaluated only on Windows 10 1607 "Anniversary Update"
	//       alias 'Redstone 1' and later versions, when value of
	//       'Size' member is 148 or greater!

	// https://msdn.microsoft.com/en-us/library/ms684179.aspx
                       LOAD_LIBRARY_SEARCH_SYSTEM32,
#else
                       0U,
#endif
                       0ULL,
                       &__security_cookie,
                       0ULL,
                       0ULL,
                       0ULL,
                       0ULL,
                       0ULL,
                       0ULL,
                       0UL};
#endif

// https://msdn.microsoft.com/en-us/library/8dbf701c.aspx
// https://msdn.microsoft.com/en-us/library/aa290051.aspx

#if _M_IX86
__declspec(naked)
VOID	__fastcall	__security_check_cookie(DWORD_PTR _stackcookie)
{
	__asm	// using x86 assembler to preserve all registers!
	{
		cmp	ecx, __security_cookie
		jne	OVERRUN
		ret
	OVERRUN:
	}
#else
VOID	__security_check_cookie(DWORD_PTR _stackcookie)
{
	if (_stackcookie == __security_cookie)
		return;
#endif
	// https://msdn.microsoft.com/en-us/library/ms680552.aspx

	RaiseException(EXCEPTION_STACK_BUFFER_OVERRUN, EXCEPTION_NONCONTINUABLE, 1UL, _AddressOfReturnAddress());

	// https://msdn.microsoft.com/en-us/library/ms682658.aspx

	ExitProcess(ERROR_STACK_BUFFER_OVERRUN);
}
#endif // GSCOOKIE

#if _MSC_VER >= 1700
__declspec(noreturn)
VOID	__report_rangecheckfailure(VOID)
{
	// https://msdn.microsoft.com/en-us/library/ms680552.aspx

	RaiseException(EXCEPTION_STACK_BUFFER_OVERRUN, EXCEPTION_NONCONTINUABLE, 1UL, _AddressOfReturnAddress());

	// https://msdn.microsoft.com/en-us/library/ms682658.aspx

	ExitProcess(ERROR_STACK_BUFFER_OVERRUN);
}
#endif

BOOL	CDECL	DebugFormatA([SA_FormatString(Style="printf")] LPCSTR lpFormat, ...)
{
	DWORD	dwOutput;
	CHAR	szOutput[1024];

	// https://msdn.microsoft.com/en-us/library/kb57fad8.aspx

	va_list	vaInput;
	va_start(vaInput, lpFormat);

	// https://msdn.microsoft.com/en-us/library/ms647551.aspx

	dwOutput = wvsprintfA(szOutput, lpFormat, vaInput);

	va_end(vaInput);

	if (dwOutput == 0UL)
#if 0
		return FALSE;
#else
		return DebugFormatA("DLLDUMMY: wvsprintfA() returned error %lu\n", GetLastError());
#endif
	szOutput[dwOutput] = '\0';

	// https://msdn.microsoft.com/en-us/library/aa363362.aspx

	OutputDebugStringA(szOutput);

	return TRUE;
}

HWINSTA	hWinSta = NULL;

WCHAR	szString[MAX_PATH] = L"";
DWORD	dwString = 0UL;

// https://msdn.microsoft.com/en-us/library/ms686892.aspx

USEROBJECTFLAGS	uof = {0};

BOOL	bInteractive = FALSE;

LPVOID	lpBuffer = NULL;

LPCWSTR	lpStrings[8] = {L"Message not available in current language or locale!",
	                L"Vulnerability and Exploit Detector Module",
	                L"<not available>",
	                L"Process Attach",
	                L"Thread Attach",
	                L"Thread Detach",
	                L"Process Detach",
	                L"<unknown>"};

LPCWSTR	lpReason = L"";

WCHAR	szCurrentDirectory[MAX_PATH] = L"";
DWORD	dwCurrentDirectory = 0UL;

WCHAR	szModule[MAX_PATH] = L"";
DWORD	dwModule = 0UL;

WCHAR	szProcess[MAX_PATH] = L"";
DWORD	dwProcess = 0UL;

#ifdef CALLER
WCHAR	szCaller[MAX_PATH] = L"";
DWORD	dwCaller = 0UL;

LPVOID	lpCallers[61] = {0};			// return addresses from stack frames
WORD	wCallers = 0U;

HMODULE	hCaller = NULL;
HMODULE	hKernelBase = NULL;
HMODULE	hKernel32 = NULL;
HMODULE	hNTDLL = NULL;
#endif

HANDLE	hToken = INVALID_HANDLE_VALUE;

struct	_TOKEN_INFO
{
	// https://msdn.microsoft.com/en-us/library/bb394727.aspx
	// https://msdn.microsoft.com/en-us/library/aa379634.aspx

	union
	{	// https://msdn.microsoft.com/en-us/library/aa379595.aspx

		SID_AND_ATTRIBUTES	Label;
		SID_AND_ATTRIBUTES	User;
	};

	// https://msdn.microsoft.com/en-us/library/aa379594.aspx

	BYTE	Sid[SECURITY_MAX_SID_SIZE];
} ti = {0};

// https://msdn.microsoft.com/en-us/library/cc962011.aspx
// https://msdn.microsoft.com/en-us/library/aa379594.aspx
// https://msdn.microsoft.com/en-us/library/aa379597.aspx
// https://msdn.microsoft.com/en-us/library/aa379598.aspx
// https://msdn.microsoft.com/en-us/library/aa379649.aspx

// S-1-5-18 = 'NT AUTHORITY\SYSTEM'
// S-1-5-19 = 'NT AUTHORITY\LocalService'
// S-1-5-20 = 'NT AUTHORITY\NetworkService'
// S-1-5-32-544 = 'BUILTIN\Administrators'
// S-1-5-32-547 = 'BUILTIN\Power Users'
// S-1-5-32-549 = 'BUILTIN\System Operators'
// S-1-5-32-551 = 'BUILTIN\Backup Operators'
// S-1-5-32-548 = 'BUILTIN\Account Operators'
// S-1-5-32-550 = 'BUILTIN\Print Operators'
// S-1-5-32-556 = 'BUILTIN\Network Configuration Operators'
// S-1-5-32-545 = 'BUILTIN\Users'
// S-1-5-32-546 = 'BUILTIN\Guests'

const	struct	_SID2
{
	BYTE	Revision;
	BYTE	SubAuthorityCount;
	SID_IDENTIFIER_AUTHORITY	IdentifierAuthority;
	DWORD	SubAuthority[2];
} sids[] = {{SID_REVISION, 1, SECURITY_NT_AUTHORITY, {SECURITY_LOCAL_SYSTEM_RID, 0UL}},
            {SID_REVISION, 1, SECURITY_NT_AUTHORITY, {SECURITY_LOCAL_SERVICE_RID, 0UL}},
            {SID_REVISION, 1, SECURITY_NT_AUTHORITY, {SECURITY_NETWORK_SERVICE_RID, 0UL}},
            {SID_REVISION, 2, SECURITY_NT_AUTHORITY, {SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS}},
            {SID_REVISION, 2, SECURITY_NT_AUTHORITY, {SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_POWER_USERS}},
            {SID_REVISION, 2, SECURITY_NT_AUTHORITY, {SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_SYSTEM_OPS}},
            {SID_REVISION, 2, SECURITY_NT_AUTHORITY, {SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_BACKUP_OPS}},
            {SID_REVISION, 2, SECURITY_NT_AUTHORITY, {SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ACCOUNT_OPS}},
            {SID_REVISION, 2, SECURITY_NT_AUTHORITY, {SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_PRINT_OPS}},
            {SID_REVISION, 2, SECURITY_NT_AUTHORITY, {SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_NETWORK_CONFIGURATION_OPS}},
            {SID_REVISION, 2, SECURITY_NT_AUTHORITY, {SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_USERS}},
            {SID_REVISION, 2, SECURITY_NT_AUTHORITY, {SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_GUESTS}}};

LPWSTR	lpSid = NULL;

// https://msdn.microsoft.com/en-us/library/aa379601.aspx

SID_NAME_USE	snu = {0};

WCHAR	szLevel[GNLEN + 1] = L"";
WCHAR	szUser[UNLEN + 1 + GNLEN + 1] = L"";

WCHAR	szAccount[UNLEN + 1] = L"";
DWORD	dwAccount = 0UL;

WCHAR	szDomain[GNLEN + 1] = L"";
DWORD	dwDomain = 0UL;

LPCWSTR	lpInserts[8] = {szProcess,
	                L"",			// command line
	                szCurrentDirectory,
	                szModule,
	                L"",			// reason
#ifdef CALLER
	                szCaller,
#else
	                L"",			// calling module (UNUSED)
#endif
	                szLevel,
	                szUser};

#ifdef EVENTLOG
HANDLE	hEventLog = NULL;
#endif

INT	nResult = 0;

#ifdef USERICON
// BUG: if MB_DEFAULT_DESKTOP_ONLY is set, MessageBoxIndirect()
//      displays no icon and fails with access violation when
//      caption or text are specified as resources!

extern	const	IMAGE_DOS_HEADER	__ImageBase;

// https://msdn.microsoft.com/en-us/library/ms645402.aspx
// https://msdn.microsoft.com/en-us/library/ms648029.aspx
// https://msdn.microsoft.com/en-us/library/dd373908.aspx

MSGBOXPARAMS	mbp = {sizeof(mbp),
		       HWND_DESKTOP,
		       (HINSTANCE) &__ImageBase,
		       MAKEINTRESOURCE(0),	// text
		       MAKEINTRESOURCE(1),	// caption
		       MB_OKCANCEL | MB_USERICON,
		       MAKEINTRESOURCE(1),	// icon
		       (DWORD_PTR) NULL,
		       (MSGBOXCALLBACK) NULL,
		       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT)};
#endif // USERICON

// https://msdn.microsoft.com/en-us/library/3hxxtd06.aspx
// https://msdn.microsoft.com/en-us/library/988ye33t.aspx
// https://msdn.microsoft.com/en-us/library/ms682583.aspx
// https://msdn.microsoft.com/en-us/library/ms682596.aspx

BOOL	WINAPI	_DllMainCRTStartup(HANDLE hModule,
		                   DWORD  dwReason,
		                   LPVOID lpReserved)
{
	DWORD	dw;
	BOOL	b;

	// https://msdn.microsoft.com/en-us/library/ms683179.aspx

	HANDLE	hProcess = GetCurrentProcess();

	// https://msdn.microsoft.com/en-us/library/ms683182.aspx

	HANDLE	hThread = GetCurrentThread();

	switch (dwReason)
	{
	case DLL_PROCESS_ATTACH:

		// called once for each process that loads this DLL

		// lpReserved is NULL if this DLL is loaded dynamically;
		// lpReserved holds the address of the CONTEXT of the initial
		// thread of the process if this DLL is loaded statically
		// (i.e. during process creation)

		// return TRUE to keep this DLL loaded or
		// return FALSE to fail loading this DLL
#ifdef GSCOOKIE
		if ((__security_cookie == 0)
		 || (__security_cookie == DEFAULT_SECURITY_COOKIE))
			__security_cookie = PRIVATE_SECURITY_COOKIE;
#endif
#ifndef THREAD
		// https://msdn.microsoft.com/en-us/library/ms682579.aspx

		if (!DisableThreadLibraryCalls(hModule))
			DebugFormatA("DLLDUMMY: DisableThreadLibraryCalls() returned error %lu\n", GetLastError());
#endif
		// https://msdn.microsoft.com/en-us/library/ms683225.aspx

		if ((hWinSta = GetProcessWindowStation()) == NULL)
			DebugFormatA("DLLDUMMY: GetProcessWindowStation() returned error %lu\n", GetLastError());
		else
			// https://msdn.microsoft.com/en-us/library/ms683238.aspx

			if (!GetUserObjectInformation(hWinSta, UOI_NAME, szString, sizeof(szString), &dwString))
				DebugFormatA("DLLDUMMY: GetUserObjectInformation() returned error %lu\n", GetLastError());
			else
				// https://msdn.microsoft.com/en-us/library/ms647489.aspx

				if (lstrcmpi(L"WinSta0", szString) == 0)

					// https://msdn.microsoft.com/en-us/library/ms683238.aspx

					if (!GetUserObjectInformation(hWinSta, UOI_FLAGS, &uof, sizeof(uof), (LPDWORD) NULL))
						DebugFormatA("DLLDUMMY: GetUserObjectInformation() returned error %lu\n", GetLastError());
					else
						if (uof.dwFlags & WSF_VISIBLE)
							bInteractive = TRUE;
#ifndef EVENTLOG
		if (!bInteractive)
			return TRUE;

		// https://msdn.microsoft.com/en-us/library/ms680345.aspx

		if (IsDebuggerPresent())
			return TRUE;
#endif
		// https://msdn.microsoft.com/en-us/library/ms683156.aspx

		lpInserts[1] = GetCommandLine();

		for (dw = 0UL; dw < sizeof(lpStrings) / sizeof(*lpStrings); dw++)

			// https://msdn.microsoft.com/en-us/library/ms647486.aspx

			if (!LoadString(hModule, dw, (LPWSTR) &lpBuffer, 0))
				DebugFormatA("DLLDUMMY: LoadString() returned error %lu\n", GetLastError());
			else
				lpStrings[dw] = lpBuffer;

		// https://msdn.microsoft.com/en-us/library/aa364934.aspx

		if ((dwCurrentDirectory = GetCurrentDirectory(sizeof(szCurrentDirectory) / sizeof(*szCurrentDirectory), szCurrentDirectory)) == 0UL)
		{
			DebugFormatA("DLLDUMMY: GetCurrentDirectory() returned error %lu\n", GetLastError());

			// https://msdn.microsoft.com/en-us/library/ms647491.aspx

			if (lstrcpyn(szCurrentDirectory, lpStrings[2], sizeof(szCurrentDirectory) / sizeof(*szCurrentDirectory)) == NULL)
				OutputDebugStringA("DLLDUMMY: lstrcpyn() returned NULL\n");
		}
		else
			if (dwCurrentDirectory > sizeof(szCurrentDirectory) / sizeof(*szCurrentDirectory))
			{
				DebugFormatA("DLLDUMMY: GetCurrentDirectory() returned %lu; length of pathname exceeds MAX_PATH\n", dwCurrentDirectory);

				// https://msdn.microsoft.com/en-us/library/ms647491.aspx

				if (lstrcpyn(szCurrentDirectory, lpStrings[2], sizeof(szCurrentDirectory) / sizeof(*szCurrentDirectory)) == NULL)
					OutputDebugStringA("DLLDUMMY: lstrcpyn() returned NULL\n");
			}
			else
				szCurrentDirectory[dwCurrentDirectory] = L'\0';

		// https://msdn.microsoft.com/en-us/library/ms683197.aspx

		if (((dwModule = GetModuleFileName(hModule, szModule, sizeof(szModule) / sizeof(*szModule))) == 0UL)
		 || (dwModule >= sizeof(szModule) / sizeof(*szModule)))
		{
			DebugFormatA("DLLDUMMY: GetModuleFileName() returned error %lu\n", GetLastError());

			// https://msdn.microsoft.com/en-us/library/ms647491.aspx

			if (lstrcpyn(szModule, lpStrings[2], sizeof(szModule) / sizeof(*szModule)) == NULL)
				OutputDebugStringA("DLLDUMMY: lstrcpyn() returned NULL\n");
		}
		else
			szModule[dwModule] = L'\0';

		// https://msdn.microsoft.com/en-us/library/ms683197.aspx

		if (((dwProcess = GetModuleFileName((HMODULE) NULL, szProcess, sizeof(szProcess) / sizeof(*szProcess))) == 0UL)
		 || (dwProcess >= sizeof(szProcess) / sizeof(*szProcess)))
		{
			DebugFormatA("DLLDUMMY: GetModuleFileName() returned error %lu\n", GetLastError());

			// https://msdn.microsoft.com/en-us/library/ms647491.aspx

			if (lstrcpyn(szProcess, lpStrings[2], sizeof(szProcess) / sizeof(*szProcess)) == NULL)
				OutputDebugStringA("DLLDUMMY: lstrcpyn() returned NULL\n");
		}
		else
			szProcess[dwProcess] = L'\0';
#ifdef CALLER
		if (lpReserved != NULL)
		{
			// https://msdn.microsoft.com/en-us/library/ms647490.aspx

			if (lstrcpy(szCaller, szProcess) == NULL)
				OutputDebugStringA("DLLDUMMY: lstrcpy() returned NULL\n");
		}
		else
		{	// https://msdn.microsoft.com/en-us/library/ms647491.aspx

			if (lstrcpyn(szCaller, lpStrings[2], sizeof(szCaller) / sizeof(*szCaller)) == NULL)
				OutputDebugStringA("DLLDUMMY: lstrcpyn() returned NULL\n");

			// https://msdn.microsoft.com/en-us/library/bb204633.aspx

			if ((wCallers = CaptureStackBackTrace(1UL,
			                                      sizeof(lpCallers) / sizeof(*lpCallers),
			                                      lpCallers,
			                                      (LPDWORD) NULL)) == 0U)
				OutputDebugStringA("DLLDUMMY: CaptureStackBackTrace() returned 0\n");
			else
			{	// https://msdn.microsoft.com/en-us/library/ms683199.aspx

				if ((hKernelBase = GetModuleHandle(L"KERNELBASE.DLL")) == NULL)
					if (GetLastError() != ERROR_MOD_NOT_FOUND)
						DebugFormatA("DLLDUMMY: GetModuleHandle() returned error %lu\n", GetLastError());

				if (((hKernel32 = GetModuleHandle(L"KERNEL32.DLL")) == NULL)
				 || ((hNTDLL = GetModuleHandle(L"NTDLL.DLL")) == NULL))
					DebugFormatA("DLLDUMMY: GetModuleHandle() returned error %lu\n", GetLastError());
				else
					for (dw = 0UL; dw < wCallers; dw++)

						// https://msdn.microsoft.com/en-us/library/ms683200.aspx

						if (!GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT,
						                       lpCallers[dw],
						                       &hCaller))
							DebugFormatA("DLLDUMMY: GetModuleHandleEx() returned error %lu\n", GetLastError());
						else
						{
							if ((hCaller == hNTDLL)
							 || (hCaller == hKernel32)
							 || (hCaller == hKernelBase))
								continue;

							// https://msdn.microsoft.com/en-us/library/ms683197.aspx

							if (((dwCaller = GetModuleFileName(hCaller, szCaller, sizeof(szCaller) / sizeof(*szCaller))) == 0UL)
							 || (dwCaller >= sizeof(szCaller) / sizeof(*szCaller)))
								DebugFormatA("DLLDUMMY: GetModuleFileName() returned error %lu\n", GetLastError());
							else
								szCaller[dwCaller] = L'\0';

							break;
						}
			}
		}
#endif // CALLER
		lpReason = lpStrings[3];
		break;

	case DLL_THREAD_ATTACH:

		// called each time a thread is created in a process that has
		// already loaded (attached to) this DLL
		// not called for the thread that loads (and attaches to) this DLL
		// and threads that exist in the process before it loaded this DLL

		// NOTE: the return value is discarded

		lpReason = lpStrings[4];
		break;

	case DLL_THREAD_DETACH:

		// same as above, but called when a thread (except the last)
		// in the process exits

		// NOTE: the return value is discarded

		lpReason = lpStrings[5];
		break;

	case DLL_PROCESS_DETACH:

		// called once for each process that unloads this DLL

		// lpReserved is NULL if this DLL is unloaded dynamically;
		// lpReserved is non-NULL if this DLL is unloaded statically
		// (i.e. during process termination)

		// NOTE: the return value is discarded

		lpReason = lpStrings[6];
		break;

//	case DLL_PROCESS_VERIFIER:

	default:
		lpReason = lpStrings[7];
	}

	lpInserts[4] = lpReason;

	// https://msdn.microsoft.com/en-us/library/ms647491.aspx

	if (lstrcpyn(szLevel, lpStrings[2], sizeof(szLevel) / sizeof(*szLevel)) == NULL)
		OutputDebugStringA("DLLDUMMY: lstrcpyn() returned NULL\n");

	if (lstrcpyn(szUser, lpStrings[2], sizeof(szUser) / sizeof(*szUser)) == NULL)
		OutputDebugStringA("DLLDUMMY: lstrcpyn() returned NULL\n");

	// https://msdn.microsoft.com/en-us/library/aa379296.aspx

	if (!OpenThreadToken(hThread, TOKEN_QUERY, FALSE, &hToken))
	{
		// https://msdn.microsoft.com/en-us/library/ms679360.aspx

		if (GetLastError() != ERROR_NO_TOKEN)
			DebugFormatA("DLLDUMMY: OpenThreadToken() returned error %lu\n", GetLastError());

		// https://msdn.microsoft.com/en-us/library/aa379295.aspx

		if (!OpenProcessToken(hProcess, TOKEN_QUERY, &hToken))
			DebugFormatA("DLLDUMMY: OpenProcessToken() returned error %lu\n", GetLastError());
	}

	if (hToken != INVALID_HANDLE_VALUE)
	{
		// https://msdn.microsoft.com/en-us/library/aa446671.aspx

		if (!GetTokenInformation(hToken, TokenIntegrityLevel, &ti, sizeof(ti), &dw))

			// https://msdn.microsoft.com/en-us/library/ms679360.aspx

			if (GetLastError() != ERROR_INVALID_PARAMETER)
				DebugFormatA("DLLDUMMY: GetTokenInformation() returned error %lu\n", GetLastError());
			else	// NT 5.x and earlier versions
			{
				for (dw = 0UL; dw < sizeof(sids) / sizeof(*sids); dw++)

					// https://msdn.microsoft.com/en-us/library/aa376389.aspx

					if (!CheckTokenMembership((HANDLE) NULL, &sids[dw], &b))
						DebugFormatA("DLLDUMMY: CheckTokenMembership() returned error %lu\n", GetLastError());
					else
						if (b)
						{
							dwAccount = sizeof(szAccount) / sizeof(*szAccount);
							dwDomain = sizeof(szDomain) / sizeof(*szDomain);

							// https://msdn.microsoft.com/en-us/library/aa379166.aspx

							if (!LookupAccountSid((LPCWSTR) NULL,
							                      &sids[dw],
							                      szAccount, &dwAccount,
							                      szDomain, &dwDomain,
							                      &snu))
								DebugFormatA("DLLDUMMY: LookupAccountSid() returned error %lu\n", GetLastError());
							else
								// https://msdn.microsoft.com/en-us/library/ms647490.aspx

								if (lstrcpy(szLevel, szAccount) == NULL)
									OutputDebugStringA("DLLDUMMY: lstrcpy() returned NULL\n");

							break;
						}
#if 0
#ifdef USERICON
				// https://msdn.microsoft.com/en-us/library/aa376389.aspx

				if (!CheckTokenMembership((HANDLE) NULL, &sids[3], &b))
					DebugFormatA("DLLDUMMY: CheckTokenMembership() returned error %lu\n", GetLastError());
				else
					if (b)
						mbp.lpszIcon = MAKEINTRESOURCE(2);
#endif
			}
		else	// NT 6.x and later versions
		{
#ifdef USERICON
			// https://msdn.microsoft.com/en-us/library/aa446657.aspx

			if (*GetSidSubAuthority(ti.Label.Sid, 0UL) >= SECURITY_MANDATORY_HIGH_RID)
				mbp.lpszIcon = MAKEINTRESOURCE(2);
#endif
#else
			}
		else	// NT 6.x and later versions
		{
#endif
			dwAccount = sizeof(szAccount) / sizeof(*szAccount);
			dwDomain = sizeof(szDomain) / sizeof(*szDomain);

			// https://msdn.microsoft.com/en-us/library/aa379166.aspx

			if (!LookupAccountSid((LPCWSTR) NULL,
			                      ti.Label.Sid,
			                      szAccount, &dwAccount,
			                      szDomain, &dwDomain,
			                      &snu))
			{
				DebugFormatA("DLLDUMMY: LookupAccountSid() returned error %lu\n", GetLastError());

				// https://msdn.microsoft.com/en-us/library/aa376399.aspx

				if (!ConvertSidToStringSid(ti.Label.Sid, &lpSid))
					DebugFormatA("DLLDUMMY: ConvertSidToStringSid() returned error %lu\n", GetLastError());
				else
				{	// https://msdn.microsoft.com/en-us/library/ms647490.aspx

					if (lstrcpy(szLevel, L"*") == NULL)
						OutputDebugStringA("DLLDUMMY: lstrcpy() returned NULL\n");
					else
						// https://msdn.microsoft.com/en-us/library/ms647487.aspx

						if (lstrcat(szLevel, lpSid) == NULL)
							OutputDebugStringA("DLLDUMMY: lstrcat() returned NULL\n");

					// https://msdn.microsoft.com/en-us/library/aa366730.aspx

					if ((lpSid = LocalFree(lpSid)) != NULL)
						DebugFormatA("DLLDUMMY: LocalFree() returned error %lu\n", GetLastError());
				}
			}
			else
				// https://msdn.microsoft.com/en-us/library/ms647490.aspx

				if (lstrcpy(szLevel, szAccount) == NULL)
					OutputDebugStringA("DLLDUMMY: lstrcpy() returned NULL\n");
		}

		// https://msdn.microsoft.com/en-us/library/aa446671.aspx
		// https://msdn.microsoft.com/en-us/library/aa379626.aspx

		if (!GetTokenInformation(hToken, TokenUser, &ti, sizeof(ti), &dw))
			DebugFormatA("DLLDUMMY: GetTokenInformation() returned error %lu\n", GetLastError());
		else
		{
			dwAccount = sizeof(szAccount) / sizeof(*szAccount);
			dwDomain = sizeof(szDomain) / sizeof(*szDomain);

			// https://msdn.microsoft.com/en-us/library/aa379166.aspx

			if (!LookupAccountSid((LPCWSTR) NULL,
			                      ti.User.Sid,
			                      szAccount, &dwAccount,
			                      szDomain, &dwDomain,
			                      &snu))
			{
				DebugFormatA("DLLDUMMY: LookupAccountSid() returned error %lu\n", GetLastError());

				// https://msdn.microsoft.com/en-us/library/aa376399.aspx

				if (!ConvertSidToStringSid(ti.User.Sid, &lpSid))
					DebugFormatA("DLLDUMMY: ConvertSidToStringSid() returned error %lu\n", GetLastError());
				else
				{	// https://msdn.microsoft.com/en-us/library/ms647490.aspx

					if (lstrcpy(szUser, L"*") == NULL)
						OutputDebugStringA("DLLDUMMY: lstrcpy() returned NULL\n");
					else
						// https://msdn.microsoft.com/en-us/library/ms647487.aspx

						if (lstrcat(szUser, lpSid) == NULL)
							OutputDebugStringA("DLLDUMMY: lstrcat() returned NULL\n");

					// https://msdn.microsoft.com/en-us/library/aa366730.aspx

					if ((lpSid = LocalFree(lpSid)) != NULL)
						DebugFormatA("DLLDUMMY: LocalFree() returned error %lu\n", GetLastError());
				}
			}
			else
				if (*szDomain == L'\0')
				{
					// https://msdn.microsoft.com/en-us/library/ms647490.aspx

					if (lstrcpy(szUser, szAccount) == NULL)
						OutputDebugStringA("DLLDUMMY: lstrcpy() returned NULL\n");
				}
				else
					// https://msdn.microsoft.com/en-us/library/ms647490.aspx

					if (lstrcpy(szUser, szDomain) == NULL)
						OutputDebugStringA("DLLDUMMY: lstrcpy() returned NULL\n");
					else
						// https://msdn.microsoft.com/en-us/library/ms647487.aspx

						if (lstrcat(szUser, L"\\") == NULL)
							OutputDebugStringA("DLLDUMMY: lstrcat() returned NULL\n");
						else
							// https://msdn.microsoft.com/en-us/library/ms647487.aspx

							if (lstrcat(szUser, szAccount) == NULL)
								OutputDebugStringA("DLLDUMMY: lstrcat() returned NULL\n");
		}

		// https://msdn.microsoft.com/en-us/library/ms724211.aspx

		if (!CloseHandle(hToken))
			DebugFormatA("DLLDUMMY: CloseHandle() returned error %lu\n", GetLastError());
	}
#ifdef EVENTLOG
	// https://msdn.microsoft.com/en-us/library/aa363678.aspx

	if ((hEventLog = RegisterEventSource((LPCWSTR) NULL, L"Vulnerability and Exploit Detector")) == NULL)
		DebugFormatA("DLLDUMMY: RegisterEventSource() returned error %lu\n", GetLastError());
	else
	{	// https://msdn.microsoft.com/en-us/library/aa363679.aspx

		if (!ReportEvent(hEventLog,
		                 EVENTLOG_ERROR_TYPE, 0U,
		                 EVT_DLLMAIN,
		                 ti.User.Sid,
#ifdef CALLER
		                 6U,
#else
		                 5U,
#endif
		                 0UL,
		                 lpInserts,
		                 NULL))
			DebugFormatA("DLLDUMMY: ReportEvent() returned error %lu\n", GetLastError());

		// https://msdn.microsoft.com/en-us/library/aa363642.aspx

		if (!DeregisterEventSource(hEventLog))
			DebugFormatA("DLLDUMMY: DeregisterEventSource() returned error %lu\n", GetLastError());
	}

	if (!bInteractive)
#if 0
		return dwReason == DLL_PROCESS_ATTACH;
#else
		return TRUE;
#endif
	// https://msdn.microsoft.com/en-us/library/ms680345.aspx

	if (IsDebuggerPresent())
#if 0
		return dwReason == DLL_PROCESS_ATTACH;
#else
		return TRUE;
#endif
#endif // EVENTLOG
	// https://msdn.microsoft.com/en-us/library/ms679351.aspx

	if (!FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_ARGUMENT_ARRAY | FORMAT_MESSAGE_FROM_HMODULE,
	                   hModule,
	                   MSG_DLLMAIN,
	                   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
	                   (LPWSTR) &lpBuffer, 0UL,
	                   lpInserts))
	{
		DebugFormatA("DLLDUMMY: FormatMessage() returned error %lu\n", GetLastError());
#ifdef USERICON
#ifdef SOUND
		// https://msdn.microsoft.com/en-us/library/ms680356.aspx

		if (!MessageBeep(MB_ICONSTOP))
			DebugFormatA("DLLDUMMY: MessageBeep() returned error %lu\n", GetLastError());
#endif
		// https://msdn.microsoft.com/en-us/library/ms645402.aspx

	//	mbp.hInstance = hModule;
		mbp.lpszText = lpReason;
		mbp.lpszCaption = lpStrings[1];

		// https://msdn.microsoft.com/en-us/library/ms645511.aspx

		if ((nResult = MessageBoxIndirect(&mbp)) == 0)
			if (GetLastError() != ERROR_SUCCESS)
				DebugFormatA("DLLDUMMY: MessageBoxIndirect() returned error %lu\n", GetLastError());
	}
	else
	{
#ifdef SOUND
		// https://msdn.microsoft.com/en-us/library/ms680356.aspx

		if (!MessageBeep(MB_ICONSTOP))
			DebugFormatA("DLLDUMMY: MessageBeep() returned error %lu\n", GetLastError());
#endif
		// https://msdn.microsoft.com/en-us/library/ms645402.aspx

	//	mbp.hInstance = hModule;
		mbp.lpszText = lpBuffer;
		mbp.lpszCaption = lpStrings[1];

		// https://msdn.microsoft.com/en-us/library/ms645511.aspx

		if ((nResult = MessageBoxIndirect(&mbp)) == 0)
			if (GetLastError() != ERROR_SUCCESS)
				DebugFormatA("DLLDUMMY: MessageBoxIndirect() returned error %lu\n", GetLastError());
#else
		// https://msdn.microsoft.com/en-us/library/ms645507.aspx

		if ((nResult = MessageBoxEx(HWND_DESKTOP,
		                            lpReason,
		                            lpStrings[1],
		                            MB_DEFAULT_DESKTOP_ONLY | MB_ICONSTOP | MB_OKCANCEL,
		                            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT))) == 0)
			if (GetLastError() != ERROR_SUCCESS)
				DebugFormatA("DLLDUMMY: MessageBoxEx() returned error %lu\n", GetLastError());
	}
	else
	{	// https://msdn.microsoft.com/en-us/library/ms645507.aspx

		if ((nResult = MessageBoxEx(HWND_DESKTOP,
		                            lpBuffer,
		                            lpStrings[1],
		                            MB_DEFAULT_DESKTOP_ONLY | MB_ICONSTOP | MB_OKCANCEL,
		                            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT))) == 0)
			if (GetLastError() != ERROR_SUCCESS)
				DebugFormatA("DLLDUMMY: MessageBoxEx() returned error %lu\n", GetLastError());
#endif // USERICON
		// https://msdn.microsoft.com/en-us/library/aa366730.aspx

		if ((lpBuffer = LocalFree(lpBuffer)) != NULL)
			DebugFormatA("DLLDUMMY: LocalFree() returned error %lu\n", GetLastError());
	}

	// the return value is evaluated only for DLL_PROCESS_ATTACH,
	// and discarded otherwise

	if (nResult != IDCANCEL)
#if 0
		return dwReason == DLL_PROCESS_ATTACH;
#else
		return TRUE;
#endif
	if (dwReason == DLL_PROCESS_DETACH)

		// https://msdn.microsoft.com/en-us/library/s975zw7k.aspx

		*(HMODULE *) _AddressOfReturnAddress() = hModule;
#ifdef THREAD
	else
		// https://msdn.microsoft.com/en-us/library/ms682579.aspx

		if (!DisableThreadLibraryCalls(hModule))
			DebugFormatA("DLLDUMMY: DisableThreadLibraryCalls() returned error %lu\n", GetLastError());
#endif
	return FALSE;
}
<<NOKEEP

DLLDUMMY.OMF: DLLDUMMY.MAK
	ML.EXE /Brepro /Cp /Cx /Fo$@ /nologo /omf /Ta<< /W3 /X
; Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

	.model	tiny

e_header struct				; DOS .EXE header
e_magic	word	5A4Dh			; magic number "MZ"
e_cblp	word	stop - header		; bytes on last page of file
e_cp	word	1			; pages in file
e_crlc	word	0			; number of relocation table entries
e_cparhdr word	sizeof header / 16	; size of header in paragraphs
e_minalloc word	0			; minimum extra paragraphs needed
e_maxalloc word	65535			; maximum extra paragraphs needed
e_ss	word	0			; initial (PSP relative) SS value
e_sp	word	stack - header		; initial (SS relative) SP value
e_csum	word	0			; checksum
e_ip	word	sizeof message		; initial (CS relative) IP value
e_cs	word	0			; initial (PSP relative) CS value
e_lfarlc word	sizeof header		; file address of relocation table
e_ovno	word	0			; overlay number
e_res	word	4 dup (0)		; reserved words
e_oemid	word	5719h			; OEM identifier
e_oeminfo word	2704h			; OEM information
e_res2	word	10 dup (0)		; reserved words
e_lfanew dword	0			; file address of new exe header
e_header ends

	.code

header	e_header <>

message	byte	"(C)opyright 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>", 0Dh, 0Ah, 07h, 24h

start:
	push	cs
	pop	ds
	xor	dx, dx
	mov	ah, 09h
	int	21h

	mov	ax, 4C01h
	int	21h
stop:
	dword	20 dup (?)
stack:
	end	start
<<NOKEEP

DLLDUMMY.DOS: DLLDUMMY.MAK
	CERTUTIL.EXE /DecodeHex /F /V << $@
4D 5A 90 00 01 00 00 00 04 00 00 00 FF FF 00 00   MZ..............
E0 00 00 00 43 00 00 00 40 00 00 00 00 00 00 00   ....C...@.......
00 00 00 00 19 57 04 27 00 00 00 00 00 00 00 00   .....W.'........
00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00   ................
28 43 29 6F 70 79 72 69 67 68 74 20 32 30 30 34   (C)opyright 2004
2D 32 30 32 34 2C 20 53 74 65 66 61 6E 20 4B 61   -2024, Stefan Ka
6E 74 68 61 6B 20 3C 73 74 65 66 61 6E 2E 6B 61   nthak <stefan.ka
6E 74 68 61 6B 40 6E 65 78 67 6F 2E 64 65 3E 0D   nthak@nexgo.de>.
0A 07 24 0E 1F 33 D2 B4 09 CD 21 B8 01 4C CD 21   .....3....!..L.!
<<NOKEEP

-DLLDUMMY.ICO: DLLDUMMY.MAK
	CERTUTIL.EXE /Decode /F /V << $@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<<NOKEEP

DLLDUMMY.RES: DLLDUMMY.MAK DLLDUMMY.ICO DLLDUMMY_GER.BIN DLLDUMMY_ENU.BIN
	RC.EXE /DUNICODE /Fo$@ /L 0 /N /R /V <<
// Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

#define STRICT

#include <winnt.h>
#include <winuser.h>
#include <winver.h>

LANGUAGE LANG_GERMAN, SUBLANG_NEUTRAL

STRINGTABLE
BEGIN
    0   "Meldung in eingestellter Sprache nicht verfgbar!"
    1   "Modul zur Demonstration und Erkennung von Schwachstellen"
    2   "<nicht verfgbar>"
    3   "mit Prozess verbinden"
    4   "mit Thread verbinden"
    5   "vom Thread trennen"
    6   "vom Prozess trennen"
    7   "<unbekannt>"
END

1 MESSAGETABLE "DLLDUMMY_GER.BIN"

LANGUAGE LANG_ENGLISH, SUBLANG_NEUTRAL

STRINGTABLE
BEGIN
    0   "Message not available in current language or locale!"
    1   "Vulnerability and Exploit Detector Module"
    2   "<not available>"
    3   "Process Attach"
    4   "Thread Attach"
    5   "Thread Detach"
    6   "Process Detach"
    7   "<unknown>"
END

1 MESSAGETABLE "DLLDUMMY_ENU.BIN"

LANGUAGE LANG_NEUTRAL, SUBLANG_NEUTRAL

1 ICON "DLLDUMMY.ICO"	// https://skanthak.hier-im-netz.de/favicon.ico

// https://msdn.microsoft.com/en-us/library/dn481241.aspx
// https://msdn.microsoft.com/en-us/library/dd371711.aspx

#ifdef ISOLATION
ISOLATIONAWARE_MANIFEST_RESOURCE_ID RT_MANIFEST
#else
CREATEPROCESS_MANIFEST_RESOURCE_ID RT_MANIFEST
#endif
BEGIN
#ifdef UTF8BOM
    // https://msdn.microsoft.com/en-us/library/dd374101.aspx

    "\xEF\xBB\xBF"
#endif
    // BUG: the module loader fails with STATUS_SXS_CANT_GEN_ACTCTX
    //      when a valid encoding other than 'UTF-8' is specified!
#ifdef UTF8XML
    "<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>\n"
#endif
    "<!-- Copyright (C) 2004-2024, Stefan Kanthak -->\n"
    "<assembly manifestVersion='1.0' xmlns='urn:schemas-microsoft-com:asm.v1'>\n"
    "    <assemblyIdentity name='eSKamation.Tools.Vulnerability and Exploit Detector Module' processorArchitecture='*' type='win32' version='1.0.16.0' />\n"
    "    <compatibility xmlns='urn:schemas-microsoft-com:compatibility.v1'>\n"
    "        <application>\n"
    "            <supportedOS Id='{e2011457-1546-43c5-a5fe-008deee3d3f0}' />\n" // NT 6.0
    "            <supportedOS Id='{35138b9a-5d96-4fbd-8e2d-a2440225f93a}' />\n" // NT 6.1
    "            <supportedOS Id='{4a2f28e3-53b9-4441-ba9c-d69d4a4a6e38}' />\n" // NT 6.2
    "            <supportedOS Id='{1f676c76-80e1-4239-95bb-83d0f6d0da78}' />\n" // NT 6.3
    "            <supportedOS Id='{8e0f7a12-bfb3-4fe8-b9a5-48fd50a15a9a}' />\n" // NT 10
    "        </application>\n"
    "    </compatibility>\n"
    "    <description>Vulnerability and Exploit Detector Module</description>\n"
#ifdef LOADFROM
    "    <file loadFrom='%SystemRoot%\\System32\\ADVAPI32.DLL' name='ADVAPI32.DLL' />\n"
    "    <file loadFrom='%SystemRoot%\\System32\\KERNEL32.DLL' name='KERNEL32.DLL' />\n"
    "    <file loadFrom='%SystemRoot%\\System32\\USER32.DLL' name='USER32.DLL' />\n"
#endif
    "</assembly>\n"
END

VERSION 0x20240107UL

// https://msdn.microsoft.com/en-us/library/aa381058.aspx
// https://msdn.microsoft.com/en-us/library/aa381049.aspx
// https://msdn.microsoft.com/en-us/library/aa381057.aspx

VS_VERSION_INFO VERSIONINFO
FILEFLAGS       VS_FF_PRIVATEBUILD | VS_FF_SPECIALBUILD
FILEFLAGSMASK   VS_FFI_FILEFLAGSMASK
FILEOS          VOS_NT_WINDOWS32
FILETYPE        VFT_DLL
FILESUBTYPE     VFT2_UNKNOWN
FILEVERSION     1,0,16,0
PRODUCTVERSION  1,0,16,0
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
#ifdef UNICODE
        BLOCK "040704B0"
#else
        BLOCK "040704E4"
#endif
        BEGIN
            VALUE "Comments",         "https://msdn.microsoft.com/de-de/library/ff919712.aspx"
            VALUE "CompanyName",      "eSKamation\n"
                                      "https://skanthak.hier-im-netz.de/minesweeper.html"
            VALUE "FileDescription",  "Modul zur Demonstration und Erkennung von Schwachstellen"
            VALUE "FileVersion",      "1.0.16.0"
            VALUE "InternalName",     "DLLDUMMY"
            VALUE "LegalCopyright",   "Copyright  2004-2024, Stefan Kanthak"
        //  VALUE "LegalTrademarks",  ""
        //  VALUE "OLESelfRegister",  "Disabled"
            VALUE "OriginalFilename", "DLLDUMMY.DLL"
            VALUE "PrivateBuild",     "NOMSVCRT"
            VALUE "ProductName",      "Vulnerability and Exploit Detector"
            VALUE "ProductVersion",   "1.0.16.0"
            VALUE "SpecialBuild",     "Release"
        END
#ifdef UNICODE
        BLOCK "040904B0"
#else
        BLOCK "040904E4"
#endif
        BEGIN
            VALUE "Comments",         "https://msdn.microsoft.com/en-us/library/ff919712.aspx"
            VALUE "CompanyName",      "eSKamation\n"
                                      "https://skanthak.hier-im-netz.de/minesweeper.html"
            VALUE "FileDescription",  "Vulnerability and Exploit Detector Module"
            VALUE "FileVersion",      "1.0.16.0"
            VALUE "InternalName",     "DLLDUMMY"
            VALUE "LegalCopyright",   "Copyright  2004-2024, Stefan Kanthak"
        //  VALUE "LegalTrademarks",  ""
        //  VALUE "OLESelfRegister",  "Disabled"
            VALUE "OriginalFilename", "DLLDUMMY.DLL"
            VALUE "PrivateBuild",     "NOMSVCRT"
            VALUE "ProductName",      "Vulnerability and Exploit Detector"
            VALUE "ProductVersion",   "1.0.16.0"
            VALUE "SpecialBuild",     "Release"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
#ifdef UNICODE
        VALUE "Translation", 0x0407, 1200, 0x0409, 1200
#else
        VALUE "Translation", 0x0407, 1252, 0x0409, 1252
#endif
    END
END
<<NOKEEP

DLLDUMMY.RC DLLDUMMY.H DLLDUMMY_GER.BIN DLLDUMMY_ENU.BIN: DLLDUMMY.MAK
	MC.EXE /b /c /n /u /v /z DLLDUMMY /U <<
;// Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

LanguageNames=(English=0x0009:ENU
               German=0x0007:GER)

SeverityNames=(Success=0x0:STATUS_SEVERITY_SUCCESS
               Informational=0x1:STATUS_SEVERITY_INFORMATIONAL
               Warning=0x2:STATUS_SEVERITY_WARNING
               Error=0x3:STATUS_SEVERITY_ERROR)

FacilityNames=(None=0x000:FACILITY_NONE)

MessageIdTypedef=DWORD

MessageId=0x0001
Facility=None
Severity=Warning
SymbolicName=EVT_DLLMAIN
Language=English
This might have been the unintended call and the execution of a dangerous or malicious executable (worm, virus, trojan horse)!%n
%n
Called module: %4!s!%n
Called entry: DllMain%n
Called function: %5!s!%n
%n
Calling module: %6!s!%n
Calling process: %1!s!%n
Calling command: %2!s!%n
%n
Current directory: %3!s!%n
.
Language=German
Dies htte der unbeabsichtigte Aufruf und die Ausfhrung eines bsartigen oder schdlichen Programms (Wurm, Virus, Trojaner) sein knnen!%n
%n
Aufgerufenes Modul: %4!s!%n
Aufgerufener Einsprungpunkt: DllMain%n
Aufgerufene Funktion: %5!s!%n
%n
Aufrufendes Modul: %6!s!%n
Aufrufender Prozess: %1!s!%n
Aufrufendes Kommando: %2!s!%n
%n
Aktuelles Verzeichnis: %3!s!%n
.

MessageId=0x0003
Facility=None
Severity=Warning
SymbolicName=MSG_DLLMAIN
Language=English
ATTENTION!
This might have been the unintended call and the execution of a dangerous or malicious executable (worm, virus, trojan horse)!

Called module: %4!s!
Called entry: DllMain
Called function: %5!s!

Calling module: %6!s!
Calling process: %1!s!
Calling command: %2!s!

Current directory: %3!s!

Privilege or integrity level: %7!s!
User account: %8!s!

See <https://support.microsoft.com/en-us/kb/2269637>
and <https://support.microsoft.com/en-us/kb/2533623>
as well as <https://msdn.microsoft.com/en-us/library/ff919712.aspx>
for more information.
.
Language=German
ACHTUNG!
Dies htte der unbeabsichtigte Aufruf und die Ausfhrung eines bsartigen oder schdlichen Programms (Wurm, Virus, Trojaner) sein knnen!

Aufgerufenes Modul: %4!s!
Aufgerufener Einsprungpunkt: DllMain
Aufgerufene Funktion: %5!s!

Aufrufendes Modul: %6!s!
Aufrufender Prozess: %1!s!
Aufrufendes Kommando: %2!s!

Aktuelles Verzeichnis: %3!s!

Privilegierungs- bzw. Integrittsstufe: %7!s!
Benutzerkennung: %8!s!

Siehe <https://support.microsoft.com/de-de/kb/2269637>
und <https://support.microsoft.com/de-de/kb/2533623>
sowie <https://msdn.microsoft.com/de-de/library/ff919712.aspx>
fr weitere Informationen.
.
<<NOKEEP UNICODE
